
#ifndef __TSTREAM_H
#define __TSTREAM_H

#include <xferprot/tbinxfer.h>
#include <crc.h>

/*

STREAM protocol
---------------
 
This is a high speed binary transfer protocol.  No acks are made.
 
Packet format:
--------------
 
If the high order two bits in the length are both one then the block
is an information block.  CRC are 16-bit cyclic-redundancy-check using
CCITT standard.  All bytes are included,
 
    End-of-file
           Length       -- 2 bytes     ** both are ZERO **
           Block number -- 1 byte
 
    Normal
 
           Length       -- 2 bytes     ** Max 49151
           Block number -- 1 byte
           Data         -- variable
           CRC          -- 2 bytes
 
    Information block
           Block type   -- 1 byte      Range $C0 - $FF
           Length       -- 1 byte
           Block number -- 1 byte
           Data         -- variable
           CRC          -- 2 bytes
 
 
Field descriptors:
------------------

Length -- Length of the block including all headers/trailers.  The
          exception to this is the EOF marker
 
Block number -- Incremented block number for all blocks starting with ZERO.
          After 255 wrap back to zero.  This is in all blocks including the
          EOF marker
 
Data   -- Any data.
 
CRC    -- CRC are 16-bit cyclic-redundancy-check using CCITT standard.
          All bytes are included,
 
All information is transmitted in NETWORK BYTE ORDER.  High order byte
is first.  This is the REVERSE of the INTEL 8088 chip.
 
 
 
Data area for Information Block
-------------------------------
 
File info -- High order byte of packet is $F0
             Data area is formatted as one or more values
                  type   -- 1 byte  -- value type
                  length -- 1 byte  -- length of value
                  value  -- n bytes -- actual value

             Value types
                  0 -- Length of file -- 4 bytes long



*/

const   byte    firstInfoType   =       0xC0;
const   byte    fileInfoType    =       0xF0;
const   byte    lastInfoType    =       0xFF;

enum	streamError { None, Timeout, Abort, WritingFile, ReadingFile, WritingPort, ReadingPort, CrcError, BlockError };

struct TStreamBlockHead
{
        enum BlockType { EOFILE, NORMAL, INFO };

        union
        {
        		byte blockType;

                struct
                {
                        word    lenght;
                        byte    number;
                } TNormalBlock;

                struct
                {
                        byte    type;
                        byte    lenght;
                        byte    number;
                } TInfoBlock;

        };
};

class TStreamXfer
:
    public TBinaryXferProtcol
{
        enum TInfoValueType { LENOFFILE };
		enum TInfoBlockType { FileInfo = fileInfoType };

        streamError lastError;

        word    theCurrentBlockSize;

        TCRC16 	theCurrentBlockCRC;

        byte theCurrentBlock; // start from 0

        Boolean getFileInfo(TStreamBlockHead& aBlock);

        Boolean decodeInfo(TStreamBlockHead& aBlock);

        Boolean sendLenOfFile();

        Boolean waitReady();
		Boolean recvBlock(word blockLen);

        Boolean recvData();
        Boolean sendData();

        virtual	Boolean receive();
        virtual	Boolean transmit();

public:
        TStreamXfer(TStream*, fstream*, xferType);

        virtual	Boolean xfer();     // vp

};

#endif

